@@ -47,8 +47,7 @@ class GitlabError(Exception):
     def __str__(self) -> str:
         if self.response_code is not None:
             return f"{self.response_code}: {self.error_message}"
-        else:
-            return f"{self.error_message}"
+        return f"{self.error_message}"
 
 
 class GitlabAuthenticationError(GitlabError):
@@ -63,6 +62,10 @@ class GitlabParsingError(GitlabError):
     pass
 
 
+class GitlabCiLintError(GitlabError):
+    pass
+
+
 class GitlabConnectionError(GitlabError):
     pass
 
@@ -83,6 +86,10 @@ class GitlabGetError(GitlabOperationError):
     pass
 
 
+class GitlabHeadError(GitlabOperationError):
+    pass
+
+
 class GitlabCreateError(GitlabOperationError):
     pass
 
@@ -107,6 +114,10 @@ class GitlabTransferProjectError(GitlabOperationError):
     pass
 
 
+class GitlabGroupTransferError(GitlabOperationError):
+    pass
+
+
 class GitlabProjectDeployKeyError(GitlabOperationError):
     pass
 
@@ -183,6 +194,14 @@ class GitlabActivateError(GitlabOperationError):
     pass
 
 
+class GitlabBanError(GitlabOperationError):
+    pass
+
+
+class GitlabUnbanError(GitlabOperationError):
+    pass
+
+
 class GitlabSubscribeError(GitlabOperationError):
     pass
 
@@ -231,6 +250,10 @@ class GitlabImportError(GitlabOperationError):
     pass
 
 
+class GitlabInvitationError(GitlabOperationError):
+    pass
+
+
 class GitlabCherryPickError(GitlabOperationError):
     pass
 
@@ -267,6 +290,10 @@ class GitlabRepairError(GitlabOperationError):
     pass
 
 
+class GitlabRestoreError(GitlabOperationError):
+    pass
+
+
 class GitlabRevertError(GitlabOperationError):
     pass
 
@@ -283,6 +310,14 @@ class GitlabUnfollowError(GitlabOperationError):
     pass
 
 
+class GitlabUserApproveError(GitlabOperationError):
+    pass
+
+
+class GitlabUserRejectError(GitlabOperationError):
+    pass
+
+
 # For an explanation of how these type-hints work see:
 # https://mypy.readthedocs.io/en/stable/generics.html#declaring-decorators
 #
@@ -311,3 +346,6 @@ def on_http_error(error: Type[Exception]) -> Callable[[__F], __F]:
         return cast(__F, wrapped_f)
 
     return wrap
+
+
+__all__ = [name for name in dir() if name.endswith("Error")]
