@@ -20,6 +20,7 @@ from typing import (
     Any,
     Callable,
     Dict,
+    Iterator,
     List,
     Optional,
     Tuple,
@@ -33,7 +34,6 @@ import requests
 import gitlab
 from gitlab import base, cli
 from gitlab import exceptions as exc
-from gitlab import types as g_types
 from gitlab import utils
 
 __all__ = [
@@ -69,7 +69,35 @@ else:
     _RestObjectBase = object
 
 
-class GetMixin(_RestManagerBase):
+class HeadMixin(_RestManagerBase):
+    @exc.on_http_error(exc.GitlabHeadError)
+    def head(
+        self, id: Optional[Union[str, int]] = None, **kwargs: Any
+    ) -> "requests.structures.CaseInsensitiveDict[Any]":
+        """Retrieve headers from an endpoint.
+
+        Args:
+            id: ID of the object to retrieve
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Returns:
+            A requests header object.
+
+        Raises:
+            GitlabAuthenticationError: If authentication is not correct
+            GitlabHeadError: If the server cannot perform the request
+        """
+        if TYPE_CHECKING:
+            assert self.path is not None
+
+        path = self.path
+        if id is not None:
+            path = f"{path}/{utils.EncodedId(id)}"
+
+        return self.gitlab.http_head(path, **kwargs)
+
+
+class GetMixin(HeadMixin, _RestManagerBase):
     _computed_path: Optional[str]
     _from_parent_attrs: Dict[str, Any]
     _obj_cls: Optional[Type[base.RESTObject]]
@@ -107,14 +135,14 @@ class GetMixin(_RestManagerBase):
         if lazy is True:
             if TYPE_CHECKING:
                 assert self._obj_cls._id_attr is not None
-            return self._obj_cls(self, {self._obj_cls._id_attr: id})
+            return self._obj_cls(self, {self._obj_cls._id_attr: id}, lazy=lazy)
         server_data = self.gitlab.http_get(path, **kwargs)
         if TYPE_CHECKING:
             assert not isinstance(server_data, requests.Response)
-        return self._obj_cls(self, server_data)
+        return self._obj_cls(self, server_data, lazy=lazy)
 
 
-class GetWithoutIdMixin(_RestManagerBase):
+class GetWithoutIdMixin(HeadMixin, _RestManagerBase):
     _computed_path: Optional[str]
     _from_parent_attrs: Dict[str, Any]
     _obj_cls: Optional[Type[base.RESTObject]]
@@ -125,9 +153,7 @@ class GetWithoutIdMixin(_RestManagerBase):
     gitlab: gitlab.Gitlab
 
     @exc.on_http_error(exc.GitlabGetError)
-    def get(
-        self, id: Optional[Union[int, str]] = None, **kwargs: Any
-    ) -> Optional[base.RESTObject]:
+    def get(self, **kwargs: Any) -> base.RESTObject:
         """Retrieve a single object.
 
         Args:
@@ -143,8 +169,6 @@ class GetWithoutIdMixin(_RestManagerBase):
         if TYPE_CHECKING:
             assert self.path is not None
         server_data = self.gitlab.http_get(self.path, **kwargs)
-        if server_data is None:
-            return None
         if TYPE_CHECKING:
             assert not isinstance(server_data, requests.Response)
             assert self._obj_cls is not None
@@ -184,7 +208,7 @@ class RefreshMixin(_RestObjectBase):
         self._update_attrs(server_data)
 
 
-class ListMixin(_RestManagerBase):
+class ListMixin(HeadMixin, _RestManagerBase):
     _computed_path: Optional[str]
     _from_parent_attrs: Dict[str, Any]
     _list_filters: Tuple[str, ...] = ()
@@ -202,20 +226,25 @@ class ListMixin(_RestManagerBase):
             all: If True, return all the items, without pagination
             per_page: Number of items to retrieve per request
             page: ID of the page to return (starts with page 1)
-            as_list: If set to False and no pagination option is
+            iterator: If set to True and no pagination option is
                 defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Returns:
-            The list of objects, or a generator if `as_list` is False
+            The list of objects, or a generator if `iterator` is True
 
         Raises:
             GitlabAuthenticationError: If authentication is not correct
             GitlabListError: If the server cannot perform the request
         """
 
-        # Duplicate data to avoid messing with what the user sent us
-        data = kwargs.copy()
+        data, _ = utils._transform_types(
+            data=kwargs,
+            custom_types=self._types,
+            transform_data=True,
+            transform_files=False,
+        )
+
         if self.gitlab.per_page:
             data.setdefault("per_page", self.gitlab.per_page)
 
@@ -226,13 +255,6 @@ class ListMixin(_RestManagerBase):
         if self.gitlab.order_by:
             data.setdefault("order_by", self.gitlab.order_by)
 
-        # We get the attributes that need some special transformation
-        if self._types:
-            for attr_name, type_cls in self._types.items():
-                if attr_name in data.keys():
-                    type_obj = type_cls(data[attr_name])
-                    data[attr_name] = type_obj.get_for_api()
-
         # Allow to overwrite the path, handy for custom listings
         path = data.pop("path", self.path)
 
@@ -241,8 +263,7 @@ class ListMixin(_RestManagerBase):
         obj = self.gitlab.http_list(path, **data)
         if isinstance(obj, list):
             return [self._obj_cls(self, item, created_from_list=True) for item in obj]
-        else:
-            return base.RESTObjectList(self, self._obj_cls, obj)
+        return base.RESTObjectList(self, self._obj_cls, obj)
 
 
 class RetrieveMixin(ListMixin, GetMixin):
@@ -254,8 +275,6 @@ class RetrieveMixin(ListMixin, GetMixin):
     _path: Optional[str]
     gitlab: gitlab.Gitlab
 
-    pass
-
 
 class CreateMixin(_RestManagerBase):
     _computed_path: Optional[str]
@@ -266,15 +285,6 @@ class CreateMixin(_RestManagerBase):
     _path: Optional[str]
     gitlab: gitlab.Gitlab
 
-    def _check_missing_create_attrs(self, data: Dict[str, Any]) -> None:
-        missing = []
-        for attr in self._create_attrs.required:
-            if attr not in data:
-                missing.append(attr)
-                continue
-        if missing:
-            raise AttributeError(f"Missing attributes: {', '.join(missing)}")
-
     @exc.on_http_error(exc.GitlabCreateError)
     def create(
         self, data: Optional[Dict[str, Any]] = None, **kwargs: Any
@@ -297,24 +307,10 @@ class CreateMixin(_RestManagerBase):
         if data is None:
             data = {}
 
-        self._check_missing_create_attrs(data)
-        files = {}
-
-        # We get the attributes that need some special transformation
-        if self._types:
-            # Duplicate data to avoid messing with what the user sent us
-            data = data.copy()
-            for attr_name, type_cls in self._types.items():
-                if attr_name in data.keys():
-                    type_obj = type_cls(data[attr_name])
-
-                    # if the type if FileAttribute we need to pass the data as
-                    # file
-                    if isinstance(type_obj, g_types.FileAttribute):
-                        k = type_obj.get_file_name(attr_name)
-                        files[attr_name] = (k, data.pop(attr_name))
-                    else:
-                        data[attr_name] = type_obj.get_for_api()
+        self._create_attrs.validate_attrs(data=data)
+        data, files = utils._transform_types(
+            data=data, custom_types=self._types, transform_data=False
+        )
 
         # Handle specific URL for creation
         path = kwargs.pop("path", self.path)
@@ -335,22 +331,6 @@ class UpdateMixin(_RestManagerBase):
     _update_uses_post: bool = False
     gitlab: gitlab.Gitlab
 
-    def _check_missing_update_attrs(self, data: Dict[str, Any]) -> None:
-        if TYPE_CHECKING:
-            assert self._obj_cls is not None
-        # Remove the id field from the required list as it was previously moved
-        # to the http path.
-        required = tuple(
-            [k for k in self._update_attrs.required if k != self._obj_cls._id_attr]
-        )
-        missing = []
-        for attr in required:
-            if attr not in data:
-                missing.append(attr)
-                continue
-        if missing:
-            raise AttributeError(f"Missing attributes: {', '.join(missing)}")
-
     def _get_update_method(
         self,
     ) -> Callable[..., Union[Dict[str, Any], requests.Response]]:
@@ -393,24 +373,13 @@ class UpdateMixin(_RestManagerBase):
         else:
             path = f"{self.path}/{utils.EncodedId(id)}"
 
-        self._check_missing_update_attrs(new_data)
-        files = {}
-
-        # We get the attributes that need some special transformation
-        if self._types:
-            # Duplicate data to avoid messing with what the user sent us
-            new_data = new_data.copy()
-            for attr_name, type_cls in self._types.items():
-                if attr_name in new_data.keys():
-                    type_obj = type_cls(new_data[attr_name])
-
-                    # if the type if FileAttribute we need to pass the data as
-                    # file
-                    if isinstance(type_obj, g_types.FileAttribute):
-                        k = type_obj.get_file_name(attr_name)
-                        files[attr_name] = (k, new_data.pop(attr_name))
-                    else:
-                        new_data[attr_name] = type_obj.get_for_api()
+        excludes = []
+        if self._obj_cls is not None and self._obj_cls._id_attr is not None:
+            excludes = [self._obj_cls._id_attr]
+        self._update_attrs.validate_attrs(data=new_data, excludes=excludes)
+        new_data, files = utils._transform_types(
+            data=new_data, custom_types=self._types, transform_data=False
+        )
 
         http_method = self._get_update_method()
         result = http_method(path, post_data=new_data, files=files, **kwargs)
@@ -493,8 +462,6 @@ class CRUDMixin(GetMixin, ListMixin, CreateMixin, UpdateMixin, DeleteMixin):
     _path: Optional[str]
     gitlab: gitlab.Gitlab
 
-    pass
-
 
 class NoUpdateMixin(GetMixin, ListMixin, CreateMixin, DeleteMixin):
     _computed_path: Optional[str]
@@ -505,8 +472,6 @@ class NoUpdateMixin(GetMixin, ListMixin, CreateMixin, DeleteMixin):
     _path: Optional[str]
     gitlab: gitlab.Gitlab
 
-    pass
-
 
 class SaveMixin(_RestObjectBase):
     """Mixin for RESTObject's that can be updated."""
@@ -657,16 +622,20 @@ class DownloadMixin(_RestObjectBase):
     def download(
         self,
         streamed: bool = False,
-        action: Optional[Callable] = None,
+        action: Optional[Callable[[bytes], None]] = None,
         chunk_size: int = 1024,
+        *,
+        iterator: bool = False,
         **kwargs: Any,
-    ) -> Optional[bytes]:
+    ) -> Optional[Union[bytes, Iterator[Any]]]:
         """Download the archive of a resource export.
 
         Args:
             streamed: If True the data will be processed by chunks of
                 `chunk_size` and each chunk is passed to `action` for
                 treatment
+            iterator: If True directly return the underlying response
+                iterator
             action: Callable responsible of dealing with chunk of
                 data
             chunk_size: Size of each chunk
@@ -685,7 +654,9 @@ class DownloadMixin(_RestObjectBase):
         )
         if TYPE_CHECKING:
             assert isinstance(result, requests.Response)
-        return utils.response_content(result, streamed, action, chunk_size)
+        return utils.response_content(
+            result, streamed, action, chunk_size, iterator=iterator
+        )
 
 
 class SubscribableMixin(_RestObjectBase):
@@ -784,7 +755,10 @@ class TimeTrackingMixin(_RestObjectBase):
         # Use the existing time_stats attribute if it exist, otherwise make an
         # API call
         if "time_stats" in self.attributes:
-            return self.attributes["time_stats"]
+            time_stats = self.attributes["time_stats"]
+            if TYPE_CHECKING:
+                assert isinstance(time_stats, dict)
+            return time_stats
 
         path = f"{self.manager.path}/{self.encoded_id}/time_stats"
         result = self.manager.gitlab.http_get(path, **kwargs)
@@ -886,8 +860,6 @@ class ParticipantsMixin(_RestObjectBase):
             all: If True, return all the items, without pagination
             per_page: Number of items to retrieve per request
             page: ID of the page to return (starts with page 1)
-            as_list: If set to False and no pagination option is
-                defined, return a generator instead of a list
             **kwargs: Extra options to send to the server (e.g. sudo)
 
         Raises:
