@@ -6,21 +6,25 @@ import gitlab
 from gitlab import cli
 from gitlab import exceptions as exc
 from gitlab import types
-from gitlab.base import RequiredOptional, RESTManager, RESTObject
+from gitlab.base import RESTManager, RESTObject
 from gitlab.mixins import CRUDMixin, ListMixin, ObjectDeleteMixin, SaveMixin
+from gitlab.types import RequiredOptional
 
 from .access_requests import GroupAccessRequestManager  # noqa: F401
 from .audit_events import GroupAuditEventManager  # noqa: F401
 from .badges import GroupBadgeManager  # noqa: F401
 from .boards import GroupBoardManager  # noqa: F401
 from .clusters import GroupClusterManager  # noqa: F401
+from .container_registry import GroupRegistryRepositoryManager  # noqa: F401
 from .custom_attributes import GroupCustomAttributeManager  # noqa: F401
 from .deploy_tokens import GroupDeployTokenManager  # noqa: F401
 from .epics import GroupEpicManager  # noqa: F401
 from .export_import import GroupExportManager, GroupImportManager  # noqa: F401
 from .group_access_tokens import GroupAccessTokenManager  # noqa: F401
 from .hooks import GroupHookManager  # noqa: F401
+from .invitations import GroupInvitationManager  # noqa: F401
 from .issues import GroupIssueManager  # noqa: F401
+from .iterations import GroupIterationManager  # noqa: F401
 from .labels import GroupLabelManager  # noqa: F401
 from .members import (  # noqa: F401
     GroupBillableMemberManager,
@@ -31,7 +35,8 @@ from .merge_requests import GroupMergeRequestManager  # noqa: F401
 from .milestones import GroupMilestoneManager  # noqa: F401
 from .notification_settings import GroupNotificationSettingsManager  # noqa: F401
 from .packages import GroupPackageManager  # noqa: F401
-from .projects import GroupProjectManager  # noqa: F401
+from .projects import GroupProjectManager, SharedProjectManager  # noqa: F401
+from .push_rules import GroupPushRulesManager
 from .runners import GroupRunnerManager  # noqa: F401
 from .statistics import GroupIssuesStatisticsManager  # noqa: F401
 from .variables import GroupVariableManager  # noqa: F401
@@ -48,7 +53,7 @@ __all__ = [
 
 
 class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
-    _short_print_attr = "name"
+    _repr_attr = "name"
 
     access_tokens: GroupAccessTokenManager
     accessrequests: GroupAccessRequestManager
@@ -64,8 +69,10 @@ class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
     exports: GroupExportManager
     hooks: GroupHookManager
     imports: GroupImportManager
+    invitations: GroupInvitationManager
     issues: GroupIssueManager
     issues_statistics: GroupIssuesStatisticsManager
+    iterations: GroupIterationManager
     labels: GroupLabelManager
     members: GroupMemberManager
     members_all: GroupMemberAllManager
@@ -74,6 +81,9 @@ class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
     notificationsettings: GroupNotificationSettingsManager
     packages: GroupPackageManager
     projects: GroupProjectManager
+    shared_projects: SharedProjectManager
+    pushrules: GroupPushRulesManager
+    registry_repositories: GroupRegistryRepositoryManager
     runners: GroupRunnerManager
     subgroups: "GroupSubgroupManager"
     variables: GroupVariableManager
@@ -95,7 +105,7 @@ class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
         path = f"/groups/{self.encoded_id}/projects/{project_id}"
         self.manager.gitlab.http_post(path, **kwargs)
 
-    @cli.register_custom_action("Group", tuple(), ("group_id",))
+    @cli.register_custom_action("Group", (), ("group_id",))
     @exc.on_http_error(exc.GitlabGroupTransferError)
     def transfer(self, group_id: Optional[int] = None, **kwargs: Any) -> None:
         """Transfer the group to a new parent group or make it a top-level group.
@@ -182,7 +192,7 @@ class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
         if provider is not None:
             path += f"/{provider}"
         path += f"/{cn}"
-        self.manager.gitlab.http_delete(path)
+        self.manager.gitlab.http_delete(path, **kwargs)
 
     @cli.register_custom_action("Group")
     @exc.on_http_error(exc.GitlabCreateError)
@@ -314,7 +324,7 @@ class GroupManager(CRUDMixin, RESTManager):
             "shared_runners_setting",
         ),
     )
-    _types = {"avatar": types.ImageAttribute, "skip_groups": types.ListAttribute}
+    _types = {"avatar": types.ImageAttribute, "skip_groups": types.ArrayAttribute}
 
     def get(self, id: Union[str, int], lazy: bool = False, **kwargs: Any) -> Group:
         return cast(Group, super().get(id=id, lazy=lazy, **kwargs))
@@ -374,7 +384,7 @@ class GroupSubgroupManager(ListMixin, RESTManager):
         "with_custom_attributes",
         "min_access_level",
     )
-    _types = {"skip_groups": types.ListAttribute}
+    _types = {"skip_groups": types.ArrayAttribute}
 
 
 class GroupDescendantGroup(RESTObject):
